package ink.wgink.login.base.controller.route;

import ink.wgink.common.component.SecurityComponent;
import ink.wgink.interfaces.menu.IMenuBaseService;
import ink.wgink.login.base.config.properties.SystemProperties;
import ink.wgink.login.base.consts.IUserCenterConst;
import ink.wgink.login.base.manager.ConfigManager;
import ink.wgink.pojo.bos.UserInfoBO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IndexRouteController
 * @Description: index
 * @Author: wanggeng
 * @Date: 2021/2/27 3:38 下午
 * @Version: 1.0
 */
@Controller
public class IndexRouteController {

    @Autowired
    private SecurityComponent securityComponent;
    @Autowired
    private SystemProperties systemProperties;
    @Autowired(required = false)
    private IMenuBaseService menuBaseService;

    @GetMapping("index")
    public ModelAndView goIndex() {
        if (!StringUtils.isBlank(systemProperties.getDefaultIndexPage())) {
            return new ModelAndView(new RedirectView(systemProperties.getDefaultHomePage()));
        }
        return new ModelAndView("forward:/default-main");
    }

    /**
     * 默认主页
     *
     * @return
     */
    @GetMapping("default-main")
    public ModelAndView defaultMain() {
        ModelAndView mv = new ModelAndView("default-main");
        UserInfoBO userInfoBO = securityComponent.getCurrentUser();
        mv.addObject("userUsername", userInfoBO.getUserUsername());
        Map<String, Object> config = ConfigManager.getInstance().getConfig();
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_TITLE)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_TITLE).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_TITLE, config.get(IUserCenterConst.SYSTEM_TITLE).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_TITLE, systemProperties.getSystemTitle());
        }
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_SUB_TITLE)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_SUB_TITLE).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_SUB_TITLE, config.get(IUserCenterConst.SYSTEM_SUB_TITLE).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_SUB_TITLE, systemProperties.getSystemSubTitle());
        }
        // 系统LOGO
        if (!Objects.isNull(config.get(IUserCenterConst.SYSTEM_LOGO)) && !StringUtils.isBlank(config.get(IUserCenterConst.SYSTEM_LOGO).toString())) {
            mv.addObject(IUserCenterConst.SYSTEM_LOGO, config.get(IUserCenterConst.SYSTEM_LOGO).toString());
        } else {
            mv.addObject(IUserCenterConst.SYSTEM_LOGO, "");
        }
        mv.addObject("ws", systemProperties.getWs());
        if (menuBaseService != null) {
            List<MenuDTO> menus = menuBaseService.listAllByParentId(IMenuBaseService.MENU_UNIFIED_USER);
            mv.addObject("menus", menus);
        }
        return mv;
    }

    /**
     * 默认导航首页
     *
     * @return
     */
    @GetMapping("default-home")
    public ModelAndView defaultHome() {
        ModelAndView mv;
        if (!StringUtils.isBlank(systemProperties.getDefaultHomePage())) {
            mv = new ModelAndView(new RedirectView(systemProperties.getDefaultHomePage()));
        } else {
            mv = new ModelAndView("default-home");
        }
        return mv;
    }

}
